  
`timescale 1ns/1ps

`define IF_DATA_WIDTH 8
   
module gw_uart_master 
 (
    clk_50M,
    rst_n,
	key2,
	tx_o,
	rx_o,
    receive_flag,
    error_flag
 );
        
 input	    clk_50M;
 input	    rst_n;
 input	    key2; 
 output     tx_o;
 input      rx_o;
 output reg receive_flag;
 output reg error_flag;
             
////////////////////////////////////////////////////////////////   
 wire                      I_TX_EN;
 wire [2:0]                I_WADDR;
 wire [`IF_DATA_WIDTH-1:0] I_WDATA;   
 wire                      I_RX_EN;  
 wire [2:0]                I_RADDR;
 wire [`IF_DATA_WIDTH-1:0] O_RDATA; 
 
 wire					   DDIS;
 wire					   INTR;
 wire					   RxRDYn;
 wire					   TxRDYn;
                               
 //wire					   DCDn;
 //wire					   CTSn;
 //wire					   DSRn;
 //wire					   RIn;
 //wire					   DTRn;
 //wire					   RTSn;
 
 wire					   tx;
 wire					   rx; 

 wire                      rst1;
 wire                      rstn;
 reg  [7:0]                delay_rst=0;
 reg  [7:0]                delay_key2=0;  
 reg  [15:0]               counter0=0;
 reg                       clk_en; 
 
 wire                      start; 
 
 wire                      receive_flag_temp;
 reg                       receive_flag_temp1; 
 
 wire                      error_flag_temp;
 reg                       error_flag_temp1;  
 
//////////////////////////////////////////////////////////////////////////
 wire                      clk_sys;
 wire                      clk_sys1; 

 Gowin_rPLL your_instance_name(
     .clkout(clk_sys), //output clkout
//	 .clkoutd(clk_sys1),
     .clkin(clk_50M) //input clkin
 );
 

 assign rst1=&{delay_rst[7],!delay_rst[6],!delay_rst[5],!delay_rst[4],!delay_rst[3],!delay_rst[2]};
 assign rstn=~rst1;
 
 assign start=&{delay_key2[7],!delay_key2[6],!delay_key2[5],!delay_key2[4],!delay_key2[3],!delay_key2[2]}; 
 

 always @(posedge clk_sys) 
    if(counter0==16'd49999) begin
	   counter0 <= 16'd0;
	clk_en <= 1'b1;
	end
	else begin
	   counter0 <= counter0 + 16'd1;
	clk_en <= 1'b0;	 
	end
    
 always @(posedge clk_sys)
    if(clk_en==1'b1) begin
       delay_rst[7:1] <= delay_rst[6:0];
       delay_rst[0] <= rst_n;
    end
	
 always @(posedge clk_sys)
    if(clk_en==1'b1) 
	begin
       delay_key2[7:1] <= delay_key2[6:0];
       delay_key2[0] <= key2;
    end	


 always @(posedge clk_sys)
    begin
       receive_flag_temp1 <= receive_flag_temp;
    end

 always @(negedge rstn or posedge clk_sys)
	if(~rstn) begin
       receive_flag <= 1'b0;
	end 
    else if(receive_flag_temp1 == 1'b0 && receive_flag_temp == 1'b1) begin 
       receive_flag <= ~receive_flag;
    end
	
	
 always @(posedge clk_sys)
    begin
       error_flag_temp1 <= error_flag_temp;
    end

 always @(negedge rstn or posedge clk_sys)
	if(~rstn) begin
       error_flag <= 1'b0;
	end
    else if(error_flag_temp1 == 1'b0 && error_flag_temp == 1'b1) begin
       error_flag <= 1'b1;
    end
	

 master_sram_uart	u_master_sram_uart
	(
      .I_CLK            (   clk_sys                ),
      .I_RESETN         (   ~rst1                  ),
	  .start            (   start                  ),
      .I_TX_EN          (   I_TX_EN                ),
      .I_WADDR          (   I_WADDR                ),
      .I_WDATA          (   I_WDATA                ),   
      .I_RX_EN          (   I_RX_EN                ),  
      .I_RADDR          (   I_RADDR                ),
      .O_RDATA          (   O_RDATA                ),
      .receive_flag     (   receive_flag_temp      ),
      .error_flag       (   error_flag_temp        )  
    );

 UART_MASTER_Top	u_UART_MASTER_Top
	(
      .I_CLK            (   clk_sys                ),
      .I_RESETN         (   ~rst1                  ),
      .I_TX_EN          (   I_TX_EN                ),
      .I_WADDR          (   I_WADDR                ),
      .I_WDATA          (   I_WDATA                ),   
      .I_RX_EN          (   I_RX_EN                ),  
      .I_RADDR          (   I_RADDR                ),
      .O_RDATA          (   O_RDATA                ),
        
	// Processor interface               
	  .DDIS   			(      				       ), // Driver disable           
	  .INTR   			(   				       ), // Interrupt                
	                                     
	// Receiver interface                
	  .SIN    			(   rx_o				   ), // Receiver serial input    
	  .RxRDYn 			(   RxRDYn				   ), // Receiver ready           
	                                     
	// Transmitter interface             
	  .SOUT   			(   tx_o				   ), // Transmitter serial output
	  .TxRDYn 			(   TxRDYn				   ), // Transmitter ready        
	                                     
	// Modem interface                   
	  .DCDn   			(   1'b0                   ), // Data Carrier Detect      
	  .CTSn   			(   1'b0                   ), // Clear To Send            
	  .DSRn   			(   1'b0                   ), // Data Set Ready           
	  .RIn    			(   1'b0                   ), // Ring Indicator           
	  .DTRn   			(                          ), // Data Terminal Ready      
	  .RTSn     		(                          ) // Request To Send          
	);
  
endmodule

