
`timescale 1ns/1ps

`define IF_DATA_WIDTH 8
   
module master_sram_uart
(
     I_CLK,
     I_RESETN,
	 start,
     I_TX_EN,
     I_WADDR,
     I_WDATA,   
     I_RX_EN,  
     I_RADDR,
     O_RDATA,
     receive_flag,
     error_flag
);

    input                       I_CLK;
    input                       I_RESETN;
    input                       start;	
    output                      I_TX_EN;
    output [2:0]                I_WADDR;
    output [`IF_DATA_WIDTH-1:0] I_WDATA;   
    output                      I_RX_EN;  
    output [2:0]                I_RADDR;
    input  [`IF_DATA_WIDTH-1:0] O_RDATA; 

    output reg                  receive_flag = 0;
    output reg                  error_flag   = 0;
 
 //////////////////////////////////////////////////////////////////////////
//	Internal Wires/Registers
    reg                         I_TX_EN;
    reg[2:0]					I_WADDR;	
    reg[`IF_DATA_WIDTH-1:0]     I_WDATA;   
    reg                         I_RX_EN;  
    reg[2:0]                    I_RADDR;
	
    reg[3:0]					wr_cntl;
    reg[3:0]					wr_reg;
    reg[3:0]					wr_index;
    reg[3:0]					wr_rd_reg;
                                
    reg[7:0]					lsr_wr;
    reg[7:0]					lsr_data;	
    reg[7:0]					rd_data;
	
    reg                         start_dl;
///////////////////////////////////////////////////////////////////////////

    always @(negedge I_RESETN or posedge I_CLK)
    if(~I_RESETN)
       start_dl <= 1'b0;	 
    else
       start_dl <= start; 

    always @(negedge I_RESETN or posedge I_CLK)
    begin
	    if(~I_RESETN)
	    begin
            I_TX_EN <= 1'b0;
            I_WADDR <= 3'b0;
            I_WDATA <= {`IF_DATA_WIDTH{1'b0}};
            I_RX_EN <= 1'b0;
            I_RADDR <= 3'b0;            
		
		    wr_index <= 0;
		    wr_cntl <= 0;
		    wr_reg <=0;
		    wr_rd_reg <= 0;
		    lsr_wr <= 0;
			lsr_data <= 0;
            rd_data <= 0;

			receive_flag <= 1'b0;			
			error_flag <= 1'b0;
	    end
	    else
		begin
		    if(wr_index==0)         // config lcr register
		    begin
		        case(wr_cntl)
		                    0:
							   if((start_dl == 1'b0) && (start == 1'b1))
		                        begin
                                   I_TX_EN <= 1'b1;
		                           I_WADDR <= 3'b011; //0x03,LCR
								   I_WDATA <= 8'h0B;
								
		                           wr_cntl <=1;
								   
								   receive_flag <= 1'b0;
								   error_flag <= 1'b0;
		                        end
							   else
							    begin
                                   I_TX_EN <= 1'b0;
								
		                           wr_cntl <=0;	

								   error_flag <= 1'b0;								   
								   receive_flag <= 1'b0;
								end   
		                    1:
		                        begin 
                                   I_TX_EN <= 1'b0;
		                           I_WADDR <= 3'b000;
								   I_WDATA <= 8'h00;
			
				                   wr_cntl <=2;
		                        end
		                    2:
		                        begin
			                      wr_index <= 1;
			                      wr_cntl <=0;
		                        end
							default:
							    begin
                                   I_TX_EN <= 1'b0;
		                           I_WADDR <= 3'b000;
								   I_WDATA <= 8'h00;	
								   wr_index <= 0;
		                           wr_cntl <=0;									   
							    end								
		            endcase 
		    end
		    else if(wr_index==1)    //read status (lsr register)
			begin
			    case(wr_rd_reg)
                            0:
			                    begin
                                   I_RX_EN <= 1'b1;
		                           I_RADDR <= 3'b101; //0x05
				            
				                   wr_rd_reg <= 1;
			                    end
			                1:
			                    begin
                                   I_RX_EN <= 1'b0;
				            
				                   wr_rd_reg <= 2; 
			                    end
			                2:
			                    begin
                                   I_RX_EN <= 1'b0;
				            
				                   wr_rd_reg <= 3; 
			                    end								
			                3:
			                    begin
				                   lsr_data <= O_RDATA; //data would be valid after two-clock delay when the rx_en is claimed
					        	   
				                   wr_rd_reg <= 4; 							   
					        	end
					        4:
					            begin
                                   if(lsr_data[6] == 1'b1) begin //if thr/tx fifo is empty, send out the data 
				                      wr_rd_reg <= 0;							   
                                      wr_index <= 2;
                                   end								  
                               	   else begin
				                      wr_rd_reg <= 0;							   
                                      wr_index <= 1;
                                   end							   
			                    end 
                            default:
                                begin
                                      I_RX_EN <= 1'b0;
                                      I_RADDR <= 3'b000;
                            	      lsr_data <= 8'h00;	
                            	      wr_index <= 0;
                                      wr_rd_reg <=0;									   
                                end
			    endcase 
			end 		
		    else if(wr_index==2)    //write data (thr register)
		    begin
		            case(wr_reg)
		                        0:
		                            begin
                                       I_TX_EN <= 1'b1;
		                               I_WADDR <= 3'b000; //0X00,THR
									   I_WDATA <= 8'h06; //send: 00000110
		
		                               wr_reg <=1;
		                            end
		                        1:
		                            begin 
                                       I_TX_EN <= 1'b0;
		                               I_WADDR <= 3'b000;
									   I_WDATA <= 8'h00;
			
				                       wr_reg <=2;
		                            end
		                        2:
		                            begin
			                           wr_index <= 3;
			                           wr_reg <=0;
		                            end
						        default:
						            begin
                                       I_TX_EN <= 1'b0;		
						        	   I_WADDR <= 3'b000; //0x00
						        	   I_WDATA <= 0;
						        	   wr_index <= 0;										  
						        	   wr_reg <= 0;
						        	end									
		            endcase 
		    end
		    else if(wr_index==3)    //read status (lsr register)
			begin
			        case(wr_rd_reg)
                        0:
			                begin
                               I_RX_EN <= 1'b1;
		                       I_RADDR <= 3'b101; //0x05
				        
				               wr_rd_reg <= 1;
			                end
			            1:
			                begin
                               I_RX_EN <= 1'b0;
					    
					           wr_rd_reg <= 2; 
			                end
			            2:
			                begin
                               I_RX_EN <= 1'b0;
					    
					           wr_rd_reg <= 3; 
			                end							
			            3:
			                begin
				               lsr_data <= O_RDATA;
							   
					           wr_rd_reg <= 4; 							   
							end
						4:
						    begin
                               if(lsr_data[0] == 1'b1) begin
					              wr_rd_reg <= 0;							   
                                  wr_index <= 4;
                               end								  
                           	   else begin
					              wr_rd_reg <= 0;							   
                                  wr_index <= 3;
                               end							   
			                end 
                		default:
                		    begin
                                  I_RX_EN <= 1'b0;
                                  I_RADDR <= 3'b000;
                			      lsr_data <= 8'h00;
                			      wr_index <= 0;
                                  wr_rd_reg <=0;									   
                		    end
			        endcase 
			end 			
		    else if(wr_index==4)    //read data (rbr register)
			begin
			        case(wr_rd_reg)
                        0:
			                begin
                               I_RX_EN <= 1'b1;
		                       I_RADDR <= 3'b000; //0x00
				        
				               wr_rd_reg <= 1;
			                end
			            1:
			                begin
                               I_RX_EN <= 1'b0;
					    
					           wr_rd_reg <= 2; 
			                end
			            2:
			                begin
                               I_RX_EN <= 1'b0;
					    
					           wr_rd_reg <= 3; 
			                end							
			            3:
			                begin
				               rd_data <= O_RDATA;
							   
					           wr_rd_reg <= 4;
							end
                        4:
                            begin		
					           wr_rd_reg <= 0;
							   
							   if(lsr_data[2] == 1'b1 || lsr_data[3] == 1'b1) begin  //if it shows rx errors, claim the error flag and return to idle
							      receive_flag <= 1'b0;
							      error_flag <= 1'b1;
								  wr_index <= 5;
							   end
                               else if(rd_data == 8'h06) begin							   
							           receive_flag <= 1'b1;
							           error_flag <= 1'b0;									   
                                       wr_index <= 0;
							   end
                               else begin
							           receive_flag <= 1'b0;
							           error_flag <= 1'b1;									   
                                       wr_index <= 0;
							   end							   
			                end 
						default:
						    begin
                                  I_RX_EN <= 1'b0;		
								  I_RADDR <= 3'b000; //0x00
								  wr_rd_reg <= 0;
								  rd_data <= 0;
								  receive_flag <=1'b0;
								  error_flag <= 1'b0;
								  wr_index <= 0;
							end 
			        endcase 
			end 
		    else if(wr_index==5)    //clear status (lsr register)
			begin			
		            case(wr_reg)
		                        0:
		                            begin
                                       I_TX_EN <= 1'b1;
		                               I_WADDR <= 3'b101; //0x05,LSR
									   I_WDATA <= 8'h0c;////write: 8'b00001100,clear FE/PE flag
									   
									   error_flag <= 1'b0;
		
		                               wr_reg <=1;
		                            end
		                        1:
		                            begin 
                                       I_TX_EN <= 1'b0;
		                               I_WADDR <= 3'b000;
									   I_WDATA <= 8'h00;
			
				                       wr_reg <=2;
		                            end
		                        2:
		                            begin
			                            wr_index <= 0;
			                            wr_reg <=0;
		                            end
						        default:
						            begin
                                          I_TX_EN <= 1'b0;		
						        		  I_WADDR <= 3'b000; //0x00
						        		  I_WDATA <= 0;
						        		  error_flag <= 1'b0;	
						        		  wr_index <= 0;										  
						        		  wr_reg <= 0;
						        	end									
		            endcase 
		    end
        end			
    end

endmodule

